; KeyISR Assembly language program.          
; by Gary Neal, Jr.  ----  [garyneal_71@yahoo.com]
;
; This is the assembly language source code for the KEYISR.OBJ file.
; The KEYISR.OBJ files allows for a low level keyboard access for QuickBasic.
;

DGROUP      group   _DATA
            assume  ds:DGROUP
_DATA       segment word public 'DATA'

keyIsPressed  db    128 dup (1)            ; 128 byte keyPress array.
keyWasPressed db    128 dup (1)            ; 128 byte wasPress array.

oldIsr9     dd    0                        ; Storage for BIOS's Interrupt.

_DATA       ends

KEYISR_TEXT segment byte public 'CODE'
            assume  cs:KEYISR_TEXT

keyisr      proc  far
            push  ax                       ; Save AX register.

            in    al,60h                   ; Get keyboard scan code.
            xor   ah,ah                    ; Zero most significant byte.

            push  ax                       ; Save keyboard scan code.
            in    al,61h                   ; Send acknowledge to keyboard.
            or    al,82h
            out   61h,al
            and   al,7Fh
            out   61h,al
            mov   al,20h                   ; Get nonspecific EOI code.
            out   20h,al                   ; Send PIC the EOI code.
            sti                            ; Reenable interrupts.
            pop   ax                       ; Restore keyboard scancode.

            or    al,al                    ; Is key pressed or released.
            jl    release                  ; If released, branch.

pressed:    and   al,7Fh                   ; Strip pressed/release flag.
            push  bx                       ; Save register BX.
            mov   bx,ax                    ; Set index to keyboard scancode.
            mov   byte ptr keyIsPressed[bx],-1  ; Set index in keyPress array.
            mov   byte ptr keyWasPressed[bx],-1 ; Set index in wasPress array.
            pop   bx                       ; Restore register BX.
            jmp   exitisr                  ; Exit the ISR.

release:    and   al,7Fh                   ; Strip pressed/release flag.
            push  bx                       ; Save register BX.
            mov   bx,ax                    ; Set index to keyboard scancode.
            mov   byte ptr keyIsPressed[bx],0   ; Set index in keyPress array.
            pop   bx                       ; Restore register BX.

exitisr:    pop   ax                       ; Restore register AX.
            iret                           ; Return from interrupt.
keyisr      endp

            public InstallKeyISR
InstallKeyISR  proc  far
            push  ax                       ; Save register AX.

            cmp   word ptr oldIsr9[2],0    ; Check Saved ISR segment address.
            jne   exitinst                 ; If not zero, exit.
            cmp   word ptr oldIsr9[0],0    ; Check Saved ISR offset address.
            jne   exitinst                 ; If not zero, exit.

            mov   ax,3509h                 ; Get Interrupt Vector 9 address.
            push  es                       ; Save Extra Segment.
            push  bx                       ; Save BX register.
            int   21h                      ; Call DOS function to get vector.
            mov   word ptr oldIsr9[2],es   ; Save Segment address of vector.
            mov   word ptr oldIsr9[0],bx   ; Save Offset address of vector.
            pop   bx                       ; Restore BX register.
            pop   es                       ; Restore Extra Segment.

            push  es                       ; Save Extra Segment.
            push  di                       ; Save Destination Index.
            push  cx                       ; Save Counter register.
            mov   cx,128                   ; Set count to 128 words.
            mov   ax,ds                    ; Get Data Segment.
            mov   es,ax                    ; Set Extra Segment to Data Segment.
            mov   di,offset keyIsPressed   ; Set Destination index to arrays.
            xor   al,al                    ; Set accumulator AX to zero.
            cld                            ; Set direction flag forward.
            rep   stosb                    ; Zero out array.
            mov   cx,128                   ; Set count to 128 words.
            mov   di,offset keyWasPressed  ; Set Destination index to arrays.
            xor   al,al                    ; Set accumulator AX to zero.
            cld                            ; Set direction flag forward.
            rep   stosb                    ; Zero out array.
            pop   cx                       ; Restore Counter register.
            pop   di                       ; Restore Destination Index.
            pop   es                       ; Restore Extra Segment.

            push  dx                       ; Save Register DX.
            push  ds                       ; Save the Data Segment.
            mov   ax,cs                    ; Get the Code Segment.
            mov   ds,ax                    ; Set Data Segment to Code Segment.
            mov   dx,offset keyisr         ; Set DX to our new keyboard ISR.
            mov   ax,2509h                 ; Set Interrupt Vector 9 address.
            int   21h                      ; Call DOS to set the new address.
            pop   ds                       ; Restore Data Segment.
            pop   dx                       ; Restore Register DX.

exitinst:   pop   ax                       ; Restore register AX.
            retf                           ; Return to BASIC.
InstallKeyISR  endp

            public RemoveKeyISR
RemoveKeyISR   proc  far
            push  ax                       ; Save register AX.

            cmp   word ptr oldIsr9[2],0    ; Check oldIsr's segment address.
            je    exitrmve                 ; If zero, exit.
            cmp   word ptr oldIsr9[0],0    ; Check oldIsr's offset address.
            je    exitrmve                 ; If zero, exit.

            push  ds                       ; Save Data Segment.
            push  dx                       ; Save DX register.
            lds   dx,oldIsr9               ; Load DS:DX with old ISR's address.
            mov   ax,2509h                 ; Set Interrupt Vector 9 address.
            int   21h                      ; Call DOS to restore old vector.
            pop   dx                       ; Restore DX register.
            pop   ds                       ; Restore Data Segment.

            mov   word ptr oldIsr9[2],0    ; Zero out oldIsr's segment address.
            mov   word ptr oldIsr9[0],0    ; Zero out oldIsr's offset address.

exitrmve:   pop   ax                       ; Restore register AX.
            retf                           ; Return to BASIC.
RemoveISR   endp

            public KeyPressed
KeyPressed  proc  far
            push  bp                       ; Save Base Pointer.
            mov   bp,sp                    ; Point it to the stack.

            push  bx                       ; Save the BX register.
            mov   ax,6[bp]                 ; Get Scancode from QuickBasic.
            and   ax,7Fh                   ; Ensure Scancode is valid.
            mov   bx,ax                    ; Set BX to Scancode for indexing.
            mov   al,byte ptr keyIsPressed[bx]  ; Get current Key State.
            or    al,byte ptr keyWasPressed[bx] ; Unite with was pressed state.
            mov   byte ptr keyWasPressed[bx],0  ; Clear the was pressed state.
            cbw                            ; Convert AL to AX for return.
            pop   bx                       ; Restore register BX.

            pop   bp                       ; Restore the Base Pointer.
            retf  2                        ; Return and clean up.
KeyPressed  endp

            public KeyPressedNow
KeyPressedNow  proc  far
            push  bp                       ; Save Base Pointer.
            mov   bp,sp                    ; Point it to the stack.

            push  bx                       ; Save the BX register.
            mov   ax,6[bp]                 ; Get Scancode from QBasic.
            and   ax,7Fh                   ; Ensure Scancode is valid.
            mov   bx,ax                    ; Set BX to Scancode for indexing.
            mov   al,byte ptr keyIsPressed[bx] ; Get current Key State.
            cbw                            ; Convert AL to AX.
            pop   bx                       ; Restore register BX.

            pop   bp                       ; Restore the Base Pointer.
            retf  2                        ; Return and clean up.
KeyPressedNow  endp

            public InitKeyISR
InitKeyISR  proc  far
            mov   word ptr oldIsr9[2],0
            mov   word ptr oldIsr9[0],0
            retf
InitKeyISR  endp

KEYISR_TEXT ends
            end
