; KeyISR Assembly language program.          
; by Gary Neal, Jr.  ----  [garyneal_71@yahoo.com]
;
; This is the assembly language source code for the KEYISR.BIN file.
; The KEYISR.BIN files allows for a low level keyboard access for QBasic.
;

          org   0                          ; Set origin to address 0.

          dw    offset setup               ; Address to the Setup procedure.
          dw    offset install             ; Address to the Install procedure.
          dw    offset remove              ; Address to the Remove procedure.
          dw    offset kpress              ; Address to the KPress procedure.
          dw    offset kpressn             ; Address to the KPressN procedure.

; Begin address that need to be change for proper operation in the
; QBasic environment.  This table is accessed by the Setup procedure
; that modifies the other procedures so that they can correctly
; access the keyboard arrays and the storage area for BIOS's interrupt
; handler.
;
; This is self-modifying code.
changes:  dw    (offset change1) + 2
          dw    (offset change2) + 2
          dw    (offset change3) + 2
          dw    (offset change4) + 2
          dw    (offset change5) + 2
          dw    (offset change6) + 2
          dw    (offset change7) + 2
          dw    (offset change8) + 1
          dw    (offset change9) + 1
          dw    (offset change10) + 2
          dw    (offset change11) + 2
          dw    (offset change12) + 2
          dw    (offset change13) + 2
          dw    (offset change14) + 2
          dw    (offset change15) + 2
          dw    (offset change16) + 2
          dw    (offset change17) + 2
          dw    (offset change18) + 2
          dw    0FFFFh

setup     proc  far

; used for debugging purposes
;          push  0000                       ; data offset
;          push  0000                       ; return address offset
;          push  0000                       ; return address segment
; end debugging

          push  bp                         ; Save the base pointer.
          mov   bp,sp                      ; Set it to the stack.

          push  ds                         ; Save BASIC's Data Segment.
          push  ax                         ; Save the AX register.
          mov   ax,cs                      ; Get the current Code Segment.
          mov   ds,ax                      ; Set Data Segment to Code Segment.

; used for debugging purposes
;          mov   word ptr [0],0100          ; data offset
; end debugging

          push  bx                         ; Save the BX register.
          mov   bx,6[bp]                   ; Get Offset parameter from QBasic.
          mov   ax,[bx]                    ; Get value of Offset into AX.
          add   ax,offset changes          ; Get pointer to Changes table.
          mov   bx,ax                      ; Set BX to that pointer.
          push  di                         ; Save the Destination Index.
loop1:    mov   ax,[bx]                    ; Get next address to be changed.
          cmp   ax,0FFFFh                  ; Last address to be changed?
          je    exitloop                   ; If last address, exit.
          push  bx                         ; If no, save Changes table pointer.
          mov   bx,6[bp]                   ; Get Offset parameter from QBasic.
          add   ax,[bx]                    ; Add Offset Value to Address.
          pop   bx                         ; Restore Changes table pointer.
          mov   di,ax                      ; Set DI to Address needing a change
          mov   ax,[di]                    ; Get value from that address.
          push  bx                         ; Save Changes table pointer.
          mov   bx,6[bp]                   ; Get Offset parameter from QBasic.
          add   ax,[bx]                    ; Add Offset to value in program.
          pop   bx                         ; Restore Changes table pointer.
          mov   [di],ax                    ; Save changes to address in program
          inc   bx                         ; Increment Changes table pointer.
          inc   bx                         ; Increment Changes table pointer.
          jmp   loop1                      ; Loop back for another.
exitloop: pop   di                         ; Restore Destination Index.
          pop   bx                         ; Restore register BX.

          pop   ax                         ; Restore Register AX.
          pop   ds                         ; Restore BASIC's Data Segment.
          pop   bp                         ; Restore Base Pointer
          retf  2                          ; Return and clean up.
setup     endp

keyisr    proc  far
          push  ds                         ; Save BASIC's data segment.
          push  ax                         ; Save AX register.

          mov   ax,cs                      ; Get current code segment.
          mov   ds,ax                      ; Set data segment to code segment.

          in    al,60h                     ; Get keyboard scan code.
          xor   ah,ah                      ; Zero most significant byte.

          push  ax                         ; Save keyboard scan code.
          in    al,61h                     ; Send acknowledge to keyboard.
          or    al,82h
          out   61h,al
          and   al,7Fh
          out   61h,al
          mov   al,20h                     ; Get nonspecific EOI code.
          out   20h,al                     ; Send PIC the EOI code.
          sti                              ; Reenable interrupts.
          pop   ax                         ; Restore keyboard scancode.

          or    al,al                      ; Is key pressed or released.
          jl    release                    ; If released, branch.

pressed:  and   al,7Fh                     ; Strip pressed/release flag.
          push  bx                         ; Save register BX.
          mov   bx,ax                      ; Set index to keyboard scancode.
change1:  mov   byte ptr keyPress[bx],-1   ; Set index in keyPress array.
change2:  mov   byte ptr wasPress[bx],-1   ; Set index in wasPress array.
          pop   bx                         ; Restore register BX.
          jmp   exitisr                    ; Exit the ISR.

release:  and   al,7Fh                     ; Strip pressed/release flag.
          push  bx                         ; Save register BX.
          mov   bx,ax                      ; Set index to keyboard scancode.
change3:  mov   byte ptr keyPress[bx],0    ; Set index in keyPress array.
          pop   bx                         ; Restore register BX.

exitisr:  pop   ax                         ; Restore register AX.
          pop   ds                         ; Restore BASIC's Data Segment.
          iret                             ; Return from interrupt.
keyisr    endp

keyPress  db    128 dup (0)                ; 128 byte keyPress array.
wasPress  db    128 dup (0)                ; 128 byte wasPress array.

install   proc  far
          push  ds                         ; Save BASIC's Data Segment.
          push  ax                         ; Save register AX.

          mov   ax,cs                      ; Get Code Segment into AX.
          mov   ds,ax                      ; Set Data Segment to Code Segment.

change4:  cmp   word ptr oldIsr[2],0       ; Check Saved ISR segment address.
          jne   exitinst                   ; If not zero, exit.
change5:  cmp   word ptr oldIsr[0],0       ; Check Saved ISR offset address.
          jne   exitinst                   ; If not zero, exit.

          mov   ax,3509h                   ; Get Interrupt Vector 9 address.
          push  es                         ; Save Extra Segment.
          push  bx                         ; Save BX register.
          int   21h                        ; Call DOS function to get vector.
change6:  mov   oldIsr[2],es               ; Save Segment address of vector.
change7:  mov   oldIsr[0],bx               ; Save Offset address of vector.
          pop   bx                         ; Restore BX register.
          pop   es                         ; Restore Extra Segment.

          push  es                         ; Save Extra Segment.
          push  di                         ; Save Destination Index.
          push  cx                         ; Save Counter register.
          mov   cx,128                     ; Set count to 128 words.
          mov   ax,ds                      ; Get Data Segment.
          mov   es,ax                      ; Set Extra Segment to Data Segment.
change8:  mov   di,offset keyPress         ; Set Destination index to arrays.
          xor   ax,ax                      ; Set accumulator AX to zero.
          cld                              ; Set direction flag forward.
          rep   stosw                      ; Zero out both arrays.
          pop   cx                         ; Restore Counter register.
          pop   di                         ; Restore Destination Index.
          pop   es                         ; Restore Extra Segment.

          push  dx                         ; Save Register DX.
change9:  mov   dx,offset keyisr           ; Set DX to our new keyboard ISR.
          mov   ax,2509h                   ; Set Interrupt Vector 9 address.
          int   21h                        ; Call DOS to set the new address.
          pop   dx                         ; Restore Register DX.

exitinst: pop   ax                         ; Restore register AX.
          pop   ds                         ; Restore BASIC's Data Segment.
          retf                             ; Return to BASIC.
install   endp

oldIsr    dd    0                          ; Old Keyboard ISR's address.

remove    proc  far
          push  ds                         ; Save BASIC's Data Segment.
          push  ax                         ; Save register AX.

          mov   ax,cs                      ; Get Code Segment into AX.
          mov   ds,ax                      ; Set Data Segment to Code Segment.

change10: cmp   word ptr oldIsr[2],0       ; Check oldIsr's segment address.
          je    exitrmve                   ; If zero, exit.
change11: cmp   word ptr oldIsr[0],0       ; Check oldIsr's offset address.
          je    exitrmve                   ; If zero, exit.

          push  ds                         ; Save Data Segment.
          push  dx                         ; Save DX register.
change12: lds   dx,oldIsr                  ; Load DS:DX with old ISR's address.
          mov   ax,2509h                   ; Set Interrupt Vector 9 address.
          int   21h                        ; Call DOS to restore old vector.
          pop   dx                         ; Restore DX register.
          pop   ds                         ; Restore Data Segment.

change13: mov   word ptr oldIsr[2],0       ; Zero out oldIsr's segment address.
change14: mov   word ptr oldIsr[0],0       ; Zero out oldIsr's offset address.

exitrmve: pop   ax                         ; Restore register AX.
          pop   ds                         ; Restore BASIC's data segment.
          retf                             ; Return to BASIC.
remove    endp

kpress    proc  far
          push  bp                         ; Save Base Pointer.
          mov   bp,sp                      ; Point it to the stack.

          push  ds                         ; Save BASIC's Data Segment.
          push  ax                         ; Save the AX register.
          mov   ax,cs                      ; Get the Code Segment.
          mov   ds,ax                      ; Set Data Segment to Code Segment.

          push  bx                         ; Save the BX register.
          mov   bx,6[bp]                   ; Get Scancode parameter from QBasic.
          mov   ax,[bx]                    ; Get Scancode into AX.
          and   ax,7Fh                     ; Ensure Scancode is valid.
          mov   bx,ax                      ; Set BX to Scancode for indexing.
change15: mov   al,byte ptr keyPress[bx]   ; Get current Key State.
change16: or    al,byte ptr wasPress[bx]   ; Unite it with was pressed state.
change17: mov   byte ptr wasPress[bx],0    ; Clear the was pressed state.
          cbw                              ; Convert AL to AX.
          mov   bx,8[bp]                   ; Get retVal parameter from QBasic.
          mov   [bx],ax                    ; Set retVal to Key's state.
          pop   bx                         ; Restore register BX.

          pop   ax                         ; Restore register AX.
          pop   ds                         ; Restore BASIC's Data Segment.
          pop   bp                         ; Restore the Base Pointer.
          retf  4                          ; Return and clean up.
kpress    endp

kpressn   proc  far
          push  bp                         ; Save Base Pointer.
          mov   bp,sp                      ; Point it to the stack.

          push  ds                         ; Save BASIC's Data Segment.
          push  ax                         ; Save the AX register.
          mov   ax,cs                      ; Get the Code Segment.
          mov   ds,ax                      ; Set Data Segment to Code Segment.


          push  bx                         ; Save the BX register.
          mov   bx,6[bp]                   ; Get Scancode parameter from QBasic.
          mov   ax,[bx]                    ; Get Scancode into AX.
          and   ax,7Fh                     ; Ensure Scancode is valid.
          mov   bx,ax                      ; Set BX to Scancode for indexing.
change18: mov   al,byte ptr keyPress[bx]   ; Get current Key State.
          cbw                              ; Convert AL to AX.
          mov   bx,8[bp]                   ; Get retVal parameter from QBasic.
          mov   [bx],ax                    ; Set retVal to Key's state.
          pop   bx                         ; Restore register BX.

          pop   ax                         ; Restore register AX.
          pop   ds                         ; Restore BASIC's Data Segment.
          pop   bp                         ; Restore the Base Pointer.
          retf  4                          ; Return and clean up.
kpressn   endp
