' QBasic Keyboard Interrupt Handler
' by Gary Neal, Jr.  ----  [garyneal_71@yahoo.com]
'
' Illustrates the use of assembly language to write a low-level
' keyboard handler for the QBasic platform.
'
' This code may run under QuickBASIC, though not recommended.
' For more info, visit my web page at:
'     http://www.geocities.com/SiliconValley/Park/7113/
'

' Keyboard handler functions.
DECLARE SUB SetupKeyISRCode ()
DECLARE SUB InstallKeyISR ()
DECLARE SUB RemoveKeyISR ()
DECLARE FUNCTION KeyPressed% (scanCode AS INTEGER)
DECLARE FUNCTION KeyPressedNow% (scanCode AS INTEGER)

' String variable that holds the machine code program.
DIM SHARED KeyISR AS STRING

' Variables for use with the test program.
DIM X AS INTEGER
DIM Y AS INTEGER
DIM delay AS LONG

' Set up and initialize the machine code program.
SetupKeyISRCode

Start:

' Initialize screen for testing.
SCREEN 13
X = 160
Y = 100
PSET (X, Y), 15

WHILE INKEY$ <> "": WEND         ' Clear the keyboard
WHILE INKEY$ = "": WEND          ' Wait for a key
WHILE INKEY$ <> "": WEND         ' Clear the keyboard

' Replace BIOS's Keyboard handler with our own.
InstallKeyISR

' Main Testing Loop
DO UNTIL KeyPressed%(1)               ' Press ESC key to exit
    PSET (X, Y), 0                    ' Erase pixel

    ' Test whether player wants to move up or down.
    IF KeyPressedNow%(72) THEN        ' Up Arrow Moves Dot Up
        Y = Y - 1                     ' Subtract one from Y-coordinate
        IF Y < 0 THEN Y = 0           ' Y-coordinate is never < 0
    ELSEIF KeyPressedNow%(80) THEN    ' Down Arrow Moves Dot Down
        Y = Y + 1                     ' Add one to Y-coordinate
        IF Y > 199 THEN Y = 199       ' Y-coordinate is never > 199
    END IF

    ' Test whether player wants to move left or right.
    IF KeyPressedNow%(75) THEN        ' Left Arrow Moves Dot Left
        X = X - 1                     ' Subtract one from X-coordinate.
        IF X < 0 THEN X = 0           ' X-coodinate is never < 0
    ELSEIF KeyPressedNow%(77) THEN    ' Right Arrow Moves Dot Right
        X = X + 1                     ' Add one to X-coordinate
        IF X > 319 THEN X = 319       ' X-coordinate is never > 319
    END IF
    PSET (X, Y), 15                   ' Redraw pixel in new position

    ' Delay program.
    ' Change the 2000 to change the speed.
    FOR delay = 1 TO 2000: NEXT delay
LOOP

' Restore BIOS's keyboard handler
RemoveKeyISR

' Restore the text screen.
SCREEN 0
WIDTH 80, 25

' Play again?
INPUT "Again (Y/N)"; A$
IF INSTR(UCASE$(A$), "Y") GOTO Start

' If not, clear the keyboard and exit.
WHILE INKEY$ <> "": WEND
END

SUB InstallKeyISR
    DIM address AS LONG
    DIM A%

    ' Setup the machine code program if not already done so.
    IF KeyISR = "" THEN
        SetupKeyISRCode
    END IF

    DEF SEG = VARSEG(KeyISR)

    ' Get the address for the Install procedure in the Machine code program.
    address = ASC(MID$(KeyISR, 3, 1))
    address = address + (ASC(MID$(KeyISR, 4, 1)) * 256)
    address = address + SADD(KeyISR)
   
    ' Compensate for short integer variables.
    IF address > 32767 THEN
        A% = INT(address - 65536)
    ELSE
        A% = INT(address)
    END IF

    ' Call the machine code procedure.
    CALL ABSOLUTE(A%)
    DEF SEG
END SUB

FUNCTION KeyPressed% (scanCode AS INTEGER)
    DIM address AS LONG
    DIM A%
    DIM retVal AS INTEGER

    ' Setup the machine code program if not already done so.
    IF KeyISR = "" THEN
        SetupKeyISRCode
    END IF

    DEF SEG = VARSEG(KeyISR)

    ' Get the address for the KPress procedure in the Machine code program.
    address = ASC(MID$(KeyISR, 7, 1))
    address = address + (ASC(MID$(KeyISR, 8, 1)) * 256)
    address = address + SADD(KeyISR)

    ' Compensate for short integer variables.
    IF address > 32767 THEN
        A% = INT(address - 65536)
    ELSE
        A% = INT(address)
    END IF

    ' Call the machine code program with the appropriate parameters.
    CALL ABSOLUTE(retVal, scanCode, A%)

    DEF SEG

    ' Set the return value for the calling program.
    KeyPressed% = retVal
END FUNCTION

FUNCTION KeyPressedNow% (scanCode AS INTEGER)
    DIM address AS LONG
    DIM A%
    DIM retVal AS INTEGER

    ' Setup the machine code program if not already done so.
    IF KeyISR = "" THEN
        SetupKeyISRCode
    END IF

    DEF SEG = VARSEG(KeyISR)

    ' Get the address for the KPressN procedure in the Machine code program.
    address = ASC(MID$(KeyISR, 9, 1))
    address = address + (ASC(MID$(KeyISR, 10, 1)) * 256)
    address = address + SADD(KeyISR)

    ' Compensate for short integer variables.
    IF address > 32767 THEN
        A% = INT(address - 65536)
    ELSE
        A% = INT(address)
    END IF

    ' Call the machine code program with the appropriate parameters.
    CALL ABSOLUTE(retVal, scanCode, A%)
    DEF SEG

    ' Set the return value for the calling program.
    KeyPressedNow% = retVal
END FUNCTION

SUB RemoveKeyISR
    DIM address AS LONG
    DIM A%

    ' Setup the machine code program if not already done so.
    IF KeyISR = "" THEN
        SetupKeyISRCode
    END IF

    DEF SEG = VARSEG(KeyISR)
   
    ' Get the address for the Remove procedure in the Machine code program.
    address = ASC(MID$(KeyISR, 5, 1))
    address = address + (ASC(MID$(KeyISR, 6, 1)) * 256)
    address = address + SADD(KeyISR)

    ' Compensate for short integer variables.
    IF address > 32767 THEN
        A% = INT(address - 65536)
    ELSE
        A% = INT(address)
    END IF

    ' Call the machine code procedure.
    CALL ABSOLUTE(A%)
    DEF SEG
END SUB

SUB SetupKeyISRCode
    DIM offset AS LONG
    DIM address AS LONG
    DIM o%, A%

    ' Load the machine code program from disk.
    OPEN "keyisr.bin" FOR BINARY ACCESS READ AS #1
    KeyISR = INPUT$(LOF(1), #1)
    CLOSE #1

    DEF SEG = VARSEG(KeyISR)

    ' Get the address of the Setup procedure in the machine code program.
    address = ASC(MID$(KeyISR, 1, 1))
    address = address + ASC(MID$(KeyISR, 2, 1)) * 256
    address = address + SADD(KeyISR)

    ' Get the offset for the variables used in the machine code program.
    offset = SADD(KeyISR)

    ' Compensate for the short integer variables.
    IF offset > 32767 THEN
        o% = INT(offset - 65536)
    ELSE
        o% = INT(offset)
    END IF

    IF address > 32767 THEN
        A% = INT(address - 65536)
    ELSE
        A% = INT(address)
    END IF

    ' Call the machine code program.
    CALL ABSOLUTE(o%, A%)
    DEF SEG
END SUB

